==> lhcsmapi-1.2.111.tar.gz: lhcsmapi-1.2.111/lhcsmapi/dbsignal/nxcals/NxcalsDbSignal.py <==
import pandas as pd
import numpy as np
from datetime import datetime
from lhcsmapi.Time import Time
from lhcsmapi.dbsignal.DbSignal import DbSignal

try:
    from cern.nxcals.pyquery.builders import *
except:
    pass


class NxcalsDbSignal(DbSignal):
    """NxcalsDbSignal class encapsulates parsing of a signal obtained from the NXCALS database.
    """

    def __init__(self, signal={}, t_start=0, t_end=0, nxcals_system=None, nxcals_device=None, nxcals_property=None,
                 spark=None):
        super().__init__('NXCALS')
        self.signal = signal
        self.t_start = t_start
        self.t_end = t_end
        self.nxcals_system = nxcals_system
        self.nxcals_device = nxcals_device
        self.nxcals_property = nxcals_property
        self.spark = spark

        if self.spark is None:
            raise Exception('You have to provide an instance of spark connector via the spark keyword')

    @staticmethod
    def build_signal(**kwargs):
        # look through kwargs for constructor arguments
        signal_name = NxcalsDbSignal.validate_key_in_kwargs('signal', str, **kwargs) or None

        t_start = NxcalsDbSignal.validate_key_in_kwargs('t_start', int, pd.Timestamp, str, float, datetime, tuple, np.datetime64, **kwargs) or 0
        t_end = NxcalsDbSignal.validate_key_in_kwargs('t_end', int, pd.Timestamp, str, float, datetime, tuple, np.datetime64, **kwargs)
        duration = NxcalsDbSignal.validate_key_in_kwargs('duration', tuple, list, **kwargs)
        tz = NxcalsDbSignal.validate_key_in_kwargs('tz', str, **kwargs)

        nxcals_system = NxcalsDbSignal.validate_key_in_kwargs('nxcals_system', str, **kwargs) or None
        nxcals_device = NxcalsDbSignal.validate_key_in_kwargs('nxcals_device', str, **kwargs) or None
        nxcals_property = NxcalsDbSignal.validate_key_in_kwargs('nxcals_property', str, **kwargs) or None

        # calculate start and end time based of parsed arguments
        t_start, t_end = Time.get_query_period_in_unix_time(t_start, t_end, duration, tz)

        # convert start and end to unix time in seconds with us precision for CALS api
        t_start_nxcals = pd.Timestamp(t_start).to_datetime64()
        t_end_nxcals = pd.Timestamp(t_end).to_datetime64()

        # different syntax since the spark instance can't be validated without being a dependency
        spark = kwargs['spark'] if 'spark' in kwargs else None

        return NxcalsDbSignal(signal_name, t_start_nxcals, t_end_nxcals, nxcals_system, nxcals_device, nxcals_property, spark)

    def read(self, signal=None, **kwargs):
        """Method returns a pandas DataFrame containing all dbsignal with an index formatted to ns
        """
        return NxcalsDbSignal.get_signal_df(self.signal, self.t_start, self.t_end,
                                          self.nxcals_system, self.nxcals_device, self.nxcals_property, self.spark)

    @staticmethod
    def get_signal_df(signal, t_start, t_end, nxcals_system, nxcals_device, nxcals_property, spark):
        if (nxcals_device is not None) and (nxcals_property is not None):
            try:
                signal_df = DevicePropertyQuery.builder(spark).system(nxcals_system) \
                    .startTime(t_start).endTime(t_end) \
                    .entity().device(nxcals_device).property(nxcals_property) \
                    .buildDataset().select('acqStamp', signal).dropna().sort("acqStamp").toPandas()
                signal_df.set_index(signal_df['acqStamp'], inplace=True)
                signal_df.drop(columns='acqStamp', inplace=True)
                return signal_df
            except:
                exception = """Querying NXCALS with the parameters signal: {0}, start time: {1}, end time: {2}, nxcals_system {3}, nxcals_device {4}, nxcals_property {5} failed
                            ___________________________________________________________________________________
                            Reproduce the error using the raw NXCALS API:\n
                            from cern.nxcals.pyquery.builders import *
                            DevicePropertyQuery.builder(spark).nxcals_system({3}) \
                    .startTime({1}).endTime({2}) \
                    .entity().nxcals_device({4}).nxcals_property({5}) \
                    .buildDataset().select('acqStamp', signal).dropna().sort("acqStamp").toPandas()
                            spark.get(\"{0}\", {1}, {2})""".format(signal, t_start, t_end, nxcals_system, nxcals_device, nxcals_property)

                raise Exception(exception)
        else:
            try:
                signal_df = VariableQuery.builder(spark)\
                    .system(nxcals_system)\
                    .startTime(t_start).endTime(t_end)\
                    .variable(signal)\
                    .buildDataset().sort("nxcals_timestamp").dropna().select('nxcals_timestamp', 'nxcals_value').toPandas()
                signal_df.set_index(signal_df['nxcals_timestamp'], inplace=True)
                signal_df.drop(columns='nxcals_timestamp', inplace=True)
                signal_df.rename(columns={'nxcals_value': signal}, inplace=True)
                return signal_df
            except:
                exception = """Querying NXCALS with the parameters signal: {0}, start time: {1}, end time: {2}, nxcals_system {3} failed
                            ___________________________________________________________________________________
                            Reproduce the error using the raw NXCALS API:\n
                            from cern.nxcals.pyquery.builders import *
                            VariableQuery.builder(spark)\
                    .system({3})\
                    .startTime({1}).endTime({2})\
                    .variable({0})\
                    .buildDataset().sort("nxcals_timestamp").dropna().select('nxcals_timestamp', 'nxcals_value').toPandas()
                            spark.get(\"{0}\", {1}, {2})""".format(signal, t_start, t_end, nxcals_system)

                raise Exception(exception)

==> lhcsmapi-1.3.229.tar.gz: lhcsmapi-1.3.229/lhcsmapi/dbsignal/nxcals/NxcalsDbSignal.py <==
import pandas as pd
import numpy as np
from datetime import datetime
from lhcsmapi.Time import Time
from lhcsmapi.dbsignal.DbSignal import DbSignal

try:
    from cern.nxcals.pyquery.builders import *
except:
    pass


class NxcalsDbSignal(DbSignal):
    """NxcalsDbSignal class encapsulates parsing of a signal obtained from the NXCALS database.
    """

    def __init__(self, signal={}, t_start=0, t_end=0, nxcals_system=None, nxcals_device=None, nxcals_property=None,
                 spark=None):
        super().__init__('NXCALS')
        self.signal = signal
        self.t_start = t_start
        self.t_end = t_end
        self.nxcals_system = nxcals_system
        self.nxcals_device = nxcals_device
        self.nxcals_property = nxcals_property
        self.spark = spark

        if self.spark is None:
            raise Exception('You have to provide an instance of spark connector via the spark keyword')

    @staticmethod
    def build_signal(**kwargs):
        # look through kwargs for constructor arguments
        signal_name = NxcalsDbSignal.validate_key_in_kwargs('signal', str, **kwargs) or None

        t_start = NxcalsDbSignal.validate_key_in_kwargs('t_start', int, pd.Timestamp, str, float, datetime, tuple, np.datetime64, **kwargs) or 0
        t_end = NxcalsDbSignal.validate_key_in_kwargs('t_end', int, pd.Timestamp, str, float, datetime, tuple, np.datetime64, **kwargs)
        duration = NxcalsDbSignal.validate_key_in_kwargs('duration', tuple, list, **kwargs)
        tz = NxcalsDbSignal.validate_key_in_kwargs('tz', str, **kwargs)

        nxcals_system = NxcalsDbSignal.validate_key_in_kwargs('nxcals_system', str, **kwargs) or None
        nxcals_device = NxcalsDbSignal.validate_key_in_kwargs('nxcals_device', str, **kwargs) or None
        nxcals_property = NxcalsDbSignal.validate_key_in_kwargs('nxcals_property', str, **kwargs) or None

        # calculate start and end time based of parsed arguments
        t_start, t_end = Time.get_query_period_in_unix_time(t_start, t_end, duration, tz)

        # convert start and end to unix time in seconds with us precision for CALS api
        t_start_nxcals = pd.Timestamp(t_start).to_datetime64()
        t_end_nxcals = pd.Timestamp(t_end).to_datetime64()

        # different syntax since the spark instance can't be validated without being a dependency
        spark = kwargs['spark'] if 'spark' in kwargs else None

        return NxcalsDbSignal(signal_name, t_start_nxcals, t_end_nxcals, nxcals_system, nxcals_device, nxcals_property, spark)

    def read(self, signal=None, **kwargs):
        """Method returns a pandas DataFrame containing all dbsignal with an index formatted to ns
        """
        return NxcalsDbSignal.get_signal_df(self.signal, self.t_start, self.t_end,
                                          self.nxcals_system, self.nxcals_device, self.nxcals_property, self.spark)

    @staticmethod
    def get_signal_df(signal, t_start, t_end, nxcals_system, nxcals_device, nxcals_property, spark):
        if (nxcals_device is not None) and (nxcals_property is not None):
            try:
                signal_df = DevicePropertyQuery.builder(spark).system(nxcals_system) \
                    .startTime(t_start).endTime(t_end) \
                    .entity().device(nxcals_device).property(nxcals_property) \
                    .buildDataset().select('acqStamp', signal).dropna().sort("acqStamp").toPandas()
                signal_df.set_index(signal_df['acqStamp'], inplace=True)
                signal_df.drop(columns='acqStamp', inplace=True)
                return signal_df
            except:
                exception = """Querying NXCALS with the parameters signal: {0}, start time: {1}, end time: {2}, nxcals_system {3}, nxcals_device {4}, nxcals_property {5} failed
                            ___________________________________________________________________________________
                            Reproduce the error using the raw NXCALS API:\n
                            from cern.nxcals.pyquery.builders import *
                            DevicePropertyQuery.builder(spark).nxcals_system(''{3}'').startTime(''{1}'').endTime(''{2}'').entity().nxcals_device(''{4}'').nxcals_property(''{5}'').buildDataset().select('acqStamp', ''signal'').dropna().sort("acqStamp").toPandas()
                            """.format(signal, t_start, t_end, nxcals_system, nxcals_device, nxcals_property)

                raise Exception(exception)
        else:
            try:
                signal_df = VariableQuery.builder(spark)\
                    .system(nxcals_system)\
                    .startTime(t_start).endTime(t_end)\
                    .variable(signal)\
                    .buildDataset().sort("nxcals_timestamp").dropna().select('nxcals_timestamp', 'nxcals_value').toPandas()
                signal_df.set_index(signal_df['nxcals_timestamp'], inplace=True)
                signal_df.drop(columns='nxcals_timestamp', inplace=True)
                signal_df.rename(columns={'nxcals_value': signal}, inplace=True)
                return signal_df
            except:
                exception = """Querying NXCALS with the parameters signal: {0}, start time: {1}, end time: {2}, nxcals_system {3} failed
                            ___________________________________________________________________________________
                            Reproduce the error using the raw NXCALS API:\n
                            from cern.nxcals.pyquery.builders import *
                            VariableQuery.builder(spark)\
                    .system({3})\
                    .startTime({1}).endTime({2})\
                    .variable({0})\
                    .buildDataset().sort("nxcals_timestamp").dropna().select('nxcals_timestamp', 'nxcals_value').toPandas()
                    """.format(signal, t_start, t_end, nxcals_system)

                raise Exception(exception)
